'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Selection = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _componentEmitter = require('component-emitter');

var _componentEmitter2 = _interopRequireDefault(_componentEmitter);

var _assert = require('./assert');

var _assert2 = _interopRequireDefault(_assert);

var _array = require('./array');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Selection = function (_Emitter) {
  _inherits(Selection, _Emitter);

  function Selection(iteratorFactory) {
    _classCallCheck(this, Selection);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Selection).call(this));

    _this.iteratorFactory = iteratorFactory;
    _this.selectedItems = [];
    _this.lastAnchor = null;
    return _this;
  }

  _createClass(Selection, [{
    key: 'toggle',
    value: function toggle(item) {
      if (this._isSelected(item)) {
        this.lastAnchor = null;
        this._removeFromSelection(item);
        item.deselect();
      } else {
        this.lastAnchor = item;
        this._addToSelection(item);
        item.select();
      }

      return this._emitChangeEvent();
    }
  }, {
    key: 'replace',
    value: function replace(item) {
      if (!this._isOnlySelectedItem(item)) {
        this.lastAnchor = item;
        for (var i = 0; i < this.selectedItems.length; i++) {
          var oldSelectedItem = this.selectedItems[i];
          oldSelectedItem.deselect();
        }

        this.selectedItems = [item];
        item.select();

        return this._emitChangeEvent();
      }
    }
  }, {
    key: 'remove',
    value: function remove(items) {
      var atLeastOneItemRemoved = false;

      for (var i = 0; i < items.length; i++) {
        var item = items[i];
        if (this._isSelected(item)) {
          this._removeFromSelection(item);
          item.deselect();
          atLeastOneItemRemoved = true;
        }
      }

      this.lastAnchor = null;
      if (atLeastOneItemRemoved) {
        return this._emitChangeEvent();
      }
    }
  }, {
    key: 'removeAll',
    value: function removeAll() {
      return this.remove(this.selectedItems.slice());
    }
  }, {
    key: 'rangeTo',
    value: function rangeTo(endItem) {
      var _this2 = this;

      var oldSelectedItems = this.selectedItems.slice();

      var startItem = this._getRangeStart();
      (0, _assert2.default)(startItem != null, 'rangeTo: no start item');

      if (this.lastAnchor != null || this.selectedItems.length > 0) {
        this.lastAnchor = startItem;
      }

      this._deselectItemsConnectedWith(startItem);

      this._performActionInRange(startItem, endItem, function (item) {
        _this2._addToSelection(item);
        return item.select();
      });

      if (!(0, _array.sameMembers)(oldSelectedItems, this.selectedItems)) {
        return this._emitChangeEvent();
      }
    }
  }, {
    key: '_getRangeStart',
    value: function _getRangeStart() {
      if (this.lastAnchor != null) {
        return this.lastAnchor;
      } else {
        if (this.selectedItems.length > 0) {
          return this._getBottommostSelectedItem();
        } else {
          var iterator = this.iteratorFactory();
          return iterator.next().value;
        }
      }
    }
  }, {
    key: '_emitChangeEvent',
    value: function _emitChangeEvent() {
      return this.emit('change', this.selectedItems.slice());
    }
  }, {
    key: '_getBottommostSelectedItem',
    value: function _getBottommostSelectedItem() {
      var iterator = this.iteratorFactory();
      var previousItem = null;

      while (true) {
        var _iterator$next = iterator.next();

        var item = _iterator$next.value;
        var done = _iterator$next.done;


        if (done) {
          return previousItem;
        } else if (this._isSelected(item)) {
          previousItem = item;
        }
      }
    }
  }, {
    key: '_performActionInRange',
    value: function _performActionInRange(startItem, endItem, action) {
      var iterator = this.iteratorFactory();
      (0, _assert2.default)(startItem != null, '_performActionInRange: no start item');
      (0, _assert2.default)(endItem != null, '_performActionInRange: no end item');

      if (startItem === endItem) {
        action(startItem);
        return;
      }

      var current = void 0;
      var item = void 0;
      var done = void 0;
      while (!(current = iterator.next()).done) {
        item = current.value;
        if (item === startItem || item === endItem) {
          break;
        }
      }

      action(item);

      var bottomOfRangeFound = false;

      while (true) {
        var next = iterator.next();
        item = next.value;
        done = next.done;
        if (done) {
          break;
        }

        action(item);

        bottomOfRangeFound = item === startItem || item === endItem;
        if (bottomOfRangeFound) {
          break;
        }
      }

      return (0, _assert2.default)(bottomOfRangeFound, '_performActionInRange: bottom of range not found');
    }
  }, {
    key: '_isOnlySelectedItem',
    value: function _isOnlySelectedItem(item) {
      return this.selectedItems.length === 1 && this._isSelected(item);
    }
  }, {
    key: '_isSelected',
    value: function _isSelected(item) {
      return this.selectedItems.indexOf(item) !== -1;
    }
  }, {
    key: '_addToSelection',
    value: function _addToSelection(item) {
      if (!this._isSelected(item)) {
        return this.selectedItems.push(item);
      }
    }
  }, {
    key: '_removeFromSelection',
    value: function _removeFromSelection(item) {
      if (this._isSelected(item)) {
        var index = this.selectedItems.indexOf(item);
        return this.selectedItems.splice(index, 1);
      }
    }
  }, {
    key: '_deselectItemsConnectedWith',
    value: function _deselectItemsConnectedWith(targetItem) {
      var iterator = this.iteratorFactory();
      var range = [];
      var isRangeWithTargetItem = false;
      var item = void 0;
      var done = void 0;

      while (true) {
        var next = iterator.next();
        item = next.value;
        done = next.done;
        if (done) {
          break;
        }

        if (this._isSelected(item)) {
          range.push(item);

          if (item === targetItem) {
            isRangeWithTargetItem = true;
          }
        } else {
          if (isRangeWithTargetItem) {
            break;
          } else {
            range = [];
          }
        }
      }

      for (var i = 0; i < range.length; i++) {
        item = range[i];
        this._removeFromSelection(item);
        item.deselect();
      }
    }
  }]);

  return Selection;
}(_componentEmitter2.default);

exports.Selection = Selection;